/**
 * @file upnp.js
 * 
 * @brief This file contain implementation of upnp protocol.
 * 
 * @author: mobile2DS
 * 
 * @date: 03.08.2013
 */

// Global constants
// Data type to distinct temp data
var UPNP_MEDIA_INFO = 'media_info';
var UPNP_POS_INFO = 'pos_info';
var UPNP_TRANSPORT_INFO = 'transport_info';
var UPNP_TRANSPORT_SETTINGS_INFO = 'transport_settings_info';
var UPNP_VOLUME_INFO = 'volume_info';

// upnp commands
var UPNP_MUTE = 'set_mute';
var UPNP_PAUSE = 'pause';
var UPNP_UNPAUSE = 'unpause';
var UPNP_RESUME = 'resume';
var UPNP_TOGGLE_PAUSE = 'toggle pause'
var UPNP_GET_MEDIA_URL = 'get_media_url';
var UPNP_GET_MEDIA_URL_POSITION = 'get_media_url_position';
var UPNP_LOAD_CONTENT_AND_PLAY = 'content_and_play';
var UPNP_LOAD_CONTENT_AND_RESUME = 'content_and_play_resume';
var UPNP_PLAY = 'play';
var UPNP_STOP = 'stop';
var UPNP_INC_VOLUME = 'inc_vol';
var UPNP_DEC_VOLUME = 'dec_vol';
var UPNP_VOLUME_STEP = 5;
var UPNP_SET_VOLUME = 'set_volume';
// for testing purposes only
var UPNP_GET_MEDIA_URL_REG_EVENT = 'get_media_reg_event';
var UPNP_STOP_RESUME_PLAY = 'stop_resume_play';
var UPNP_RESUME_ALARM = 'resume_alarm';

var SEEK_ATTEMPTS = 50;
var SEEK_TEUFEL_DELAY = 800; // ms

/*
 * ------------------------------------------------------------------------
 * COMMON methods
 */

function cleanXMLText(sXML) {
  return sXML.replace(/&(lt|gt|quot|amp;apos);/g, function(m, t) {
                            switch (t) {
                              case 'lt':
                                return '<';
                              case 'gt':
                                return '>';
                              case 'quot':   
                                return '\"';
                              case 'amp;apos':
                                return "\'";
                              default:
                               return "";
                            }
                         }); 
} 
 
function htmlUnescape(str)
{
  if (typeof(str) === "string")
    return str.replace(/&amp;/g, '&').replace(/&amp;/g, '&').replace(/&quot;/g, '"').replace(/&lt;/g, '<').replace(/&gt;/g, '>').replace(/&#039;/g, '\'').replace(/&apos;/g, "'");
  else
    return str;
}

function xmlEscape(str) {
    return str.replace(/&/g, '&amp;')
              .replace(/</g, '&lt;')
              .replace(/>/g, '&gt;')
              .replace(/"/g, '&quot;')
              .replace(/'/g, '&apos;');
  };

function xmlEscapeURI(str) {
    return str.replace(/&/g, '&amp;amp;')
              .replace(/</g, '&lt;')
              .replace(/>/g, '&gt;')
};
 
function sleep(delay) {
	var start = new Date().getTime();
	while (new Date().getTime() < start + delay)
		;
}

function getPlaylistFromText(sString) {
	var oArray=sString.split(';');
	oArray=oArray.splice(0,oArray.length-1);
	lWarn.logln('URL: ' + oArray.join(';'));
	return oArray.join(';')
}

function getVolumeFromText(sString) {
	var oArray=sString.split(';');
	lWarn.logln('volumne: ' + oArray[oArray.length-1]);
	return parseInt(oArray[oArray.length-1],10);
}


/**
 * @brief Save temp data
 * 
 * @param oTempData -
 *            holds temp data for saving in format: 1.) player id 2.) action id
 *            3.) action type: present == 1, or activity == 0 4.) data type:
 *            custom defined 5.) array holding values
 * 
 * @return Returns true if new mode for room is assigned, else false
 */
function saveTempData(nPlayerID, nActionID, nActionType, szDataType, oTempData) {
	try {
		var tmp_data_root = 'upnp/tempData';
		// Check if same temp data already exists

		// Write new temp data
		var path_prefix = nPlayerID + '/' + szDataType;

		// if (!bTempDataExists)
		{
			// Store player ID
			storeSubParameter(tmp_data_root, path_prefix + '/player_id', nPlayerID);
			// Store action ID
			storeSubParameter(tmp_data_root, path_prefix + '/action_id', nActionID);
			// Store action type (present == 1, activity == 0)
			storeSubParameter(tmp_data_root, path_prefix + '/action_type', nActionType);

			for (i = 0; i < oTempData.length; i++) {
				lInfo.logln(i + ': ' + oTempData[i]);
				storeSubParameter(tmp_data_root, path_prefix + '/temp' + i, oTempData[i]);
			}

			Property.store();

			lInfo.logln("Temp data stored");
		}
	} catch (e) {
		lWarn.logln("Error by saving temp data: " + e);
		for ( var i in e) {
			lWarn.logln(i + ': ' + e[i]);
		}
	}
}

function loadTempData(nPlayerID, nActionID, nActionType, szDataType, oTempData) {
	var idValue = -1;
	var tmp_data_root = 'upnp/tempData';
	var path_appendix = nPlayerID + '/' + szDataType;
	var full_path = tmp_data_root + '/' + path_appendix;

	lInfo.logln('Search for temp data in: ' + full_path);

	var baseNode = Property.getNode(full_path);
	if (baseNode != null) {
		var oArrayNode = baseNode.getChildren();
		if ((oArrayNode != null) && (oArrayNode.length > 3)) {
			lInfo.logln(oArrayNode[1].getValue());

			if ((oArrayNode[0].getValue() == nPlayerID) && (oArrayNode[1].getValue() == nActionID) && (oArrayNode[2].getValue() == nActionType)) {
				lInfo.logln('Fill output array with temp values');
				// fill the array
				for ( var i = 0; i < oArrayNode.length - 3; i++) {
					oTempData.push(oArrayNode[i + 3].getValue());
				}
			}
		} else {
			lWarn.logln('Read array is not valid');
		}
	}

	return idValue;
}

/**
 * @brief Parse UPNP XML response and return value for specific field
 * 
 * @param szXMLRespond -
 *            respond from upnp device in XML syntax
 * @param szXMLFieldName -
 *            searched field name enclosing the value
 * @param szContainsText -
 *            filter that found value must contain this text. if '' no filter
 *            applying
 * 
 * @return
 */
function upnpParseXMLRespond(szXMLRespond, szXMLFieldName, szContainsText) {
	// lInfo.logln(szXMLRespond);
	var searched_value = '';
	var found = 0;
	var xml_text = '';
	var start_pos = 0;

	xml_text = szXMLRespond;

	do {
		start_pos = xml_text.indexOf('<' + szXMLFieldName);

		if (start_pos == -1) {
			break;
		}

		start_pos += szXMLFieldName.length + 1;
		start_pos += 1 /* skip '>' */;

		var end_pos = xml_text.indexOf('</' + szXMLFieldName);

		if (end_pos == -1) {
			break;
		}
		;

		searched_value = xml_text.substring(start_pos, end_pos);

		if (szContainsText != "") {
			if (searched_value.indexOf(szContainsText) == -1) {
				start_pos = end_pos + szXMLFieldName.length + 1;
				xml_text = xml_text.substring(start_pos, xml_text.length);

				lInfo.logln("Filter not match, continue search: : " + xml_text);
			} else {
				found = 1;
			}
		} else {
			found = 1;
		}
	} while ((szContainsText != "") && (found == 0) && (xml_text != ""))

	lInfo.logln("XML parser found: " + searched_value);

	return searched_value;
}

function upnpResumeMusic(szIp, nPort, nValue, nPlayerId, szAVTransportService, oCurlOptions, sResumeFile, sSeek) {
	var result = new Object({
		'status' : '',
		'data' : ''
	});

	// resume music from playlist
	lInfo.logln("upnp-controller after timeout:" + sResumeFile);
	if (sResumeFile != "") {
		upnpSetContentSrc(nPlayerId, szIp, nPort, szAVTransportService, sResumeFile, result, oCurlOptions);
		upnpPlay(nPlayerId, szIp, nPort, szAVTransportService, result, oCurlOptions);
		lInfo.logln("upnp-controller seek:" + sSeek);

		if (sSeek != "") {
			lInfo.logln("upnp-controller inside seek:" + sSeek);
			var bSeek = false;
			var nAttemps = SEEK_ATTEMPTS;
			while (!bSeek) {
				sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, szAVTransportService, nValue, result, oCurlOptions);
				lInfo.logln("upnp-controller attempt:" + nAttemps + " / " + sInfo);
				if (sInfo == 'PLAYING') {
					upnpSeek(nPlayerId, szIp, nPort, sSeek, szAVTransportService, result, oCurlOptions);
					bSeek = true;
				}

				nAttemps = nAttemps - 1;
				if (!bSeek && nAttemps < 1)
					bSeek = true;
				sleep(200);
			}
		}
	} else { // stop
		upnpStop(nPlayerId, szIp, nPort, szAVTransportService, result, oCurlOptions);
	}

	// reseting scene priority
	lInfo.logln("reseting scene priority on resume play!");
	storeSubParameter('upnp', 'activeScenePriority', 0);
	Property.store();
}

function upnpResumeMusicRaumfeld(szIp, nPort, nValue, nPlayerId, szAVTransportService, oCurlOptions, sResumeFile, sSeek, sResumeFileXML, nSeekTrack,nVolumne) {
    var result = new Object({
        'status' : '',
        'data' : ''
    });

    // resume music from playlist
    lInfo.logln("upnp-controller after timeout:" + sResumeFile);
    lInfo.logln("upnp-controller after timeout sSeek:" + sSeek);
    lInfo.logln("upnp-controller after timeout nSeekTrack:" + nSeekTrack);
    lInfo.logln("upnp-controller after timeout xml:" + sResumeFileXML);
    if (sResumeFile.length > 0) {
        if (sResumeFileXML.length > 0)
          sResumeFile = sResumeFileXML;
        upnpSetContentSrc(nPlayerId, szIp, nPort, szAVTransportService, sResumeFile, result, oCurlOptions);
        if (nSeekTrack > 1) { // track selected, reposition to track inside current playlist 
          lInfo.logln("Inside track seek");
          upnpStop(nPlayerId, szIp, nPort, szAVTransportService, result, oCurlOptions);
          
          timeoutHandle = setTimeout(function() {
                lInfo.logln("Seek to track3="+nSeekTrack);
                upnpSeekToTrack(nPlayerId, szIp, nPort, nSeekTrack, szAVTransportService, result, oCurlOptions);  
                timeoutHandle2 = setTimeout(function() {
                   lInfo.logln("Seek to position3="+sSeek);
                   upnpSeek(nPlayerId, szIp, nPort, sSeek, szAVTransportService, result, oCurlOptions);
                }, SEEK_TEUFEL_DELAY);// a longer delay is required to work on teufel speakers
            }, SEEK_TEUFEL_DELAY); // must delay for a while if we want that seek is executed
        }
        
        if (sSeek != "") {
            timeoutHandle = setTimeout(function() {
                upnpSeek(nPlayerId, szIp, nPort, sSeek, szAVTransportService, result, oCurlOptions);
            }, SEEK_TEUFEL_DELAY); // must delay for a while if we want that seek is executed
        } 
    } else { // stop
        upnpStop(nPlayerId, szIp, nPort, szAVTransportService, result, oCurlOptions);
    }

    // reseting scene priority
    lInfo.logln("reseting scene priority on resume play!");
    storeSubParameter('upnp', 'activeScenePriority', 0);
    Property.store();
}


/*
 * ---------------------------------------------------------------------------
 * UPNP related METHODS
 */

/**
 * @brief Execute UPNP command
 * 
 * @param
 * 
 * @return
 */

function executeUpnpCmd(szCmd, szDeviceType, szIp, nPort, nValue, szMediaContentURL, nPlayerId, szMediaContentXML) {
	var result = new Object({
		'status' : '',
		'data' : '',
        'mediaInfo' : ''
	});

	var curl_options = new Object({
		'debug' : 1,
		'connectTimeout' : 10
	/* in seconds */
	});

	lInfo.logln("executeUpnpCmd: " + szCmd + " player id: " + nPlayerId + " ip addr: " + szIp + " port: " + nPort + " device type: " + szDeviceType);
	var avtransport_service_type = '/AVTransport/ctrl';
	if (szDeviceType == 'SONOS') {
		avtransport_service_type = '/MediaRenderer/AVTransport/Control';
	} else if (szDeviceType == 'SAMSUNGTV') {
		avtransport_service_type = '/AVTransport';
	} else if (szDeviceType == 'BUBBLE') {
		avtransport_service_type = '/AVTransport/action';
	} else if (szDeviceType == 'generic') {
		avtransport_service_type = Property.getNode('upnp-players/'+nPlayerId+'/avControlURL').getValue();
    } else if (szDeviceType == 'RAUMFELD') {
        avtransport_service_type = '/TransportService/Control';
    }

	var render_service_type = '/RenderingControl/ctrl';
	if (szDeviceType == 'SONOS') {
		render_service_type = '/MediaRenderer/RenderingControl/Control';
	} else if (szDeviceType == 'SAMSUNGTV') {
		render_service_type = '/RenderingControl';
	} else if (szDeviceType == 'BUBBLE') {
		render_service_type = '/RenderingControl/action';
	} else if (szDeviceType == 'generic') {
		render_service_type = Property.getNode('upnp-players/'+nPlayerId+'/renderURL').getValue();
    } else if (szDeviceType == 'BUBBLE') {
        render_service_type = '/RenderingService/Control';
    } else if (szDeviceType == 'RAUMFELD') {
        render_service_type = '/RenderingService/Control';
    }

	if (szCmd == UPNP_MUTE) {
		lInfo.logln("upnp-controller exec setMute");
		upnpSetMute(nPlayerId, szIp, nPort, nValue, render_service_type, result, curl_options);
		// upnpSetMute1(ip, port, value);
	} else if (szCmd == UPNP_TOGGLE_PAUSE) {	
		sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
		if (sInfo == "PLAYING") {
            if (szDeviceType != 'RAUMFELD') {
			    oCurrent = upnpGetPositionInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result);
			    storeSubParameter('activity', nPlayerId + '/pauseTrack', oCurrent.track);
			    storeSubParameter('activity', nPlayerId + '/pauseTrackPos', oCurrent.position);
			    Property.store();
            }
			upnpSetPause(nPlayerId, szIp, nPort, nValue, avtransport_service_type, result, curl_options);
		} else {
			lInfo.logln("upnp-controller exec unpause");
            if (szDeviceType != 'RAUMFELD') {
			    var sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
			    upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);

			    var sResumeFrom = loadSubParameter('activity', nPlayerId + '/pauseTrack');
			    var sResumeSeek = loadSubParameter('activity', nPlayerId + '/pauseTrackPos');
			    if (sResumeFrom != '') {
				    var timeoutHandle;
				    timeoutHandle = loadSubParameter('upnp', 'timer');
				    lInfo.logln("UnpauseFrom:" + sResumeFrom);
				    timeoutHandle = setTimeout(function() {
					    upnpResumeMusic(szIp, nPort, nValue, nPlayerId, avtransport_service_type, curl_options, sResumeFrom, sResumeSeek)
				    }, nValue);

				    // Store timeoutHandle to the property
				    storeSubParameter('upnp', 'timer', timeoutHandle);
				    Property.store();
			    } else {
				    // only load previous music
				    if (szMediaContentURL.split(';').length>1)
				    {                  
					    upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, getVolumeFromText(szMediaContentURL), result, curl_options);
					    upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, getPlaylistFromText(szMediaContentURL), result, curl_options);
				    } else
					    upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, szMediaContentURL, result, curl_options);
			    }
			    // clear remembered pause panic event state
			    storeSubParameter('activity', nPlayerId + '/pauseTrack', '');
			    storeSubParameter('activity', nPlayerId + '/pauseTrackPos', '');
			    Property.store();			
            }
            else { // RAUMFELD unpause
              upnpPlay(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);  
            }
		}		
	} else if (szCmd == UPNP_PAUSE) {
		lInfo.logln("upnp-controller exec Pause");
		if (szDeviceType != 'RAUMFELD') {
            if (nValue == 1) { // alarms - remember current playsong
			    sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
			    if (sInfo == "PLAYING") {
				    oCurrent = upnpGetPositionInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result);
				    storeSubParameter('activity', nPlayerId + '/pauseTrack', oCurrent.track);
				    storeSubParameter('activity', nPlayerId + '/pauseTrackPos', oCurrent.position);
				    Property.store();
				    lInfo.logln("Pause - remembering current track info: " + oCurrent.track);
			    }
		    }
        }
        
		upnpSetPause(nPlayerId, szIp, nPort, nValue, avtransport_service_type, result, curl_options);
	} else if (szCmd == UPNP_RESUME) {
		lInfo.logln("upnp-controller exec Resume");
		upnpSetResume(nPlayerId, szIp, nPort, nValue, avtransport_service_type, result, curl_options);
	} else if (szCmd == UPNP_STOP) {
		lInfo.logln("upnp-controller exec Stop - defVolume" + nValue);
		upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
		if (nValue > -1)
			upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, nValue, result, curl_options);
	} else if (szCmd == UPNP_LOAD_CONTENT_AND_PLAY) {
		lInfo.logln("upnp-controller exec LoadContentAndPlay");
		if (nValue == 1) { // alarms - remember current playsong
			sPlay = Property.getProperty("/scripts/upnp-controller/activity/" + nPlayerId + "/lastAlarmTrack");
			lInfo.logln("sPlay="+sPlay);
            if (sPlay == '' || sPlay == null) { // store only if no other event has stored
				// before this
				sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
				oCurrent = upnpGetPositionInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result);
                var sXmlPlay = "";
                if (szDeviceType == 'RAUMFELD') {
                    var resultMedia = new Object({
                            'status' : '',
                            'data' : '',
                            'dataXML' : ''    
                          });                
                    upnpGetMediaContent(nPlayerId, szIp, nPort, avtransport_service_type, resultMedia, curl_options);    
                    oCurrent.track = resultMedia.data;
                    if (resultMedia.dataXML.indexOf("audioBroadcast.radio")>-1) {// only for radio
                        sXmlPlay = resultMedia.dataXML;
                    }
                }
                var volume = upnpGetVolume(nPlayerId, szIp, nPort, render_service_type, result, curl_options);
                
				storeSubParameter('activity', nPlayerId + '/lastAlarmTrack', oCurrent.track);
                storeSubParameter('activity', nPlayerId + '/lastAlarmTrackXML', sXmlPlay);
                storeSubParameter('activity', nPlayerId + '/lastAlarmTrackNr', oCurrent.trackNR);
				storeSubParameter('activity', nPlayerId + '/lastAlarmTrackPos', oCurrent.position);
				storeSubParameter('activity', nPlayerId + '/lastAlarmTrackState', sInfo); // PLAYING,
                storeSubParameter('activity', nPlayerId + '/lastAlarmTrackVolume', volume); 
				// STOPPED...
				Property.store();
				lInfo.logln("Store LastAlarm " + oCurrent.track);
			}
		}
		upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);

        var sPlayUrl = szMediaContentURL; 
        
        lInfo.logln("sPlayUrl="+sPlayUrl);
        var nVolume = -1;
        if (szMediaContentURL.split(';').length>1)
        {
            nVolume = getVolumeFromText(szMediaContentURL);
            sPlayUrl = getPlaylistFromText(szMediaContentURL); 
        } 
        lInfo.logln("nVolume="+nVolume);
        lInfo.logln("sPlayUrl="+sPlayUrl);
        lInfo.logln("szMediaContentXML="+szMediaContentXML);

        if (nVolume > -1) {
            upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, nVolume, result, curl_options);
        }
        
        if (szDeviceType != 'RAUMFELD') {
  		    upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, sPlayUrl, result, curl_options);
            upnpPlay(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
        }
        else {
            // parsing track number from from playlist
            var nSeekTrack = -1;
            if (sPlayUrl.indexOf('track:')==0) {
              var seekTrack = sPlayUrl.substring(sPlayUrl.indexOf("track:")+6,sPlayUrl.indexOf("*@*"));  
              var tmpTrack = parseInt(seekTrack);
              if (tmpTrack != 'NaN')  
                nSeekTrack = parseInt(tmpTrack);
                
              sPlayUrl = sPlayUrl.substring(sPlayUrl.indexOf("*@*")+3);
            }
            // for playing radiostations we need to send xml over SetAvTransportURI
            if (szMediaContentXML != "" && szMediaContentXML != "null") {
              sPlayUrl = szMediaContentXML;  
            }
            
            lInfo.logln("sPlayUrl="+sPlayUrl);
            upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, sPlayUrl, result, curl_options);
            if (nSeekTrack > 1) { // track selected, reposition to track inside current playlist 
              upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);  
              timeoutHandle = setTimeout(function() {
                    upnpSeekToTrack(nPlayerId, szIp, nPort, nSeekTrack, avtransport_service_type, result, curl_options);  
                }, SEEK_TEUFEL_DELAY); // must delay for a while if we want that seek is executed
            }
        }
        
	} else if (szCmd == UPNP_UNPAUSE) {
		lInfo.logln("upnp-controller exec unpause");
		if (szDeviceType != 'RAUMFELD') {
            var sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
		    upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);

		    var sResumeFrom = loadSubParameter('activity', nPlayerId + '/pauseTrack');
		    var sResumeSeek = loadSubParameter('activity', nPlayerId + '/pauseTrackPos');
		    if (sResumeFrom != '') {
			    var timeoutHandle;
			    timeoutHandle = loadSubParameter('upnp', 'timer');
			    lInfo.logln("UnpauseFrom:" + sResumeFrom);
			    timeoutHandle = setTimeout(function() {
				    upnpResumeMusic(szIp, nPort, nValue, nPlayerId, avtransport_service_type, curl_options, sResumeFrom, sResumeSeek)
			    }, nValue);

			    // Store timeoutHandle to the property
			    storeSubParameter('upnp', 'timer', timeoutHandle);
			    Property.store();
		    } else {
			    // only load previous music
			    var szMediaURL=szMediaContentURL;
			    if (szMediaContentURL.split(';').length>1)
			    {
				    upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, getVolumeFromText(szMediaContentURL), result, curl_options);
				    upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, getPlaylistFromText(szMediaContentURL), result, curl_options);
			    } else
				    upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, szMediaContentURL, result, curl_options);
		    }
		    // clear remembered pause panic event state
		    storeSubParameter('activity', nPlayerId + '/pauseTrack', '');
		    storeSubParameter('activity', nPlayerId + '/pauseTrackPos', '');
		    Property.store();
        }
        else
          upnpPlay(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);  
	} 
    else if (szCmd == UPNP_RESUME_ALARM) {
		lInfo.logln("upnp-controller exec Resume from undo alarm");
		var sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
		// if (sInfo == 'PLAYING')
		upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);

		var sResumeFrom = loadSubParameter('activity', nPlayerId + '/lastAlarmTrackPos');
		var sResumeState = loadSubParameter('activity', nPlayerId + '/lastAlarmTrackState');
        var nTrackNr = loadSubParameter('activity', nPlayerId + '/lastAlarmTrackNr');
        var nLastVolume = loadSubParameter('activity', nPlayerId + '/lastAlarmTrackVolume');
		if (sResumeState == 'PLAYING') {
			lInfo.logln("Set the timeout interval when playlist must be resumed");
			var timeoutHandle;
			// Fetch timeoutHandle from the property
			timeoutHandle = loadSubParameter('upnp', 'timer');

			lInfo.logln("ResumeFrom:" + sResumeFrom);
            
            var sPlayUrl = szMediaContentURL; 
            
            lInfo.logln("sPlayUrl="+sPlayUrl);
            var nVolume = -1;
            if (szMediaContentURL.split(';').length>1)
            {
                nVolume = getVolumeFromText(szMediaContentURL);
                sPlayUrl = getPlaylistFromText(szMediaContentURL); 
            }
            else
              nVolume = nLastVolume; 
            
            if (szDeviceType != 'RAUMFELD') {
			    timeoutHandle = setTimeout(function() {
				    if (nVolume>1) {
					    upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, nVolume, result, curl_options);
                    }
					upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, sPlayUrl, result, curl_options);
			    }, nValue);
            } 
            else {
                timeoutHandle = setTimeout(function() {
                    if (nVolume>1) {
                        upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, nVolume, result, curl_options);
                    }
                    upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, sPlayUrl, result, curl_options);
                    if (nTrackNr > 1) { // track selected, reposition to track inside current playlist 
                        upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);  
                        setTimeout(function() {
                            upnpSeekToTrack(nPlayerId, szIp, nPort, nTrackNr, avtransport_service_type, result, curl_options);  
                            setTimeout(function() {
                               lInfo.logln("Seek to position="+sResumeFrom);
                               upnpSeek(nPlayerId, szIp, nPort, sResumeFrom, avtransport_service_type, result, curl_options);
                            }, SEEK_TEUFEL_DELAY);// a longer delay is required to work on teufel speakers
                        }, SEEK_TEUFEL_DELAY); // must delay for a while if we want that seek is executed
                    }else {
                        setTimeout(function() {
                           lInfo.logln("Seek to position="+sResumeFrom);
                           upnpSeek(nPlayerId, szIp, nPort, sResumeFrom, avtransport_service_type, result, curl_options);
                        }, SEEK_TEUFEL_DELAY);// a longer delay is required to work on teufel speakers
                    }
                }, nValue);
            }
            
			// Store timeoutHandle to the property
			storeSubParameter('upnp', 'timer', timeoutHandle);
			Property.store();
		} else {
			// only load previous music
			if (szMediaContentURL.split(';').length>1)
			{
				upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, getVolumeFromText(szMediaContentURL), result, curl_options);
				upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, getPlaylistFromText(szMediaContentURL), result, curl_options);
			} else
				upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, szMediaContentURL, result, curl_options);
		}
		// clear remembered resum panic event state
		storeSubParameter('activity', nPlayerId + '/lastAlarmTrack', '');
        storeSubParameter('activity', nPlayerId + '/lastAlarmTrackXML', '');
        storeSubParameter('activity', nPlayerId + '/lastAlarmTrackNr', '');
		storeSubParameter('activity', nPlayerId + '/lastAlarmTrackPos', '');
		storeSubParameter('activity', nPlayerId + '/lastAlarmTrackState', '');
		Property.store();
	} 
    else if (szCmd == UPNP_LOAD_CONTENT_AND_RESUME) {
		lInfo.logln("upnp-controller exec=" + UPNP_LOAD_CONTENT_AND_RESUME);
		var sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
		var sResumeFile = "";
        var sResumeFileXML = "";
		var sSeek = "";
        var nTrackNr = -1;
		if (sInfo == 'PLAYING') {
			// /upnpSeek(nPlayerId, szIp, nPort, 888, avtransport_service_type,
			// 'TRACK_NR');
			var oCurrent = upnpGetPositionInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
			var volume = upnpGetVolume(nPlayerId, szIp, nPort, render_service_type, result, curl_options);
                        
			if (oCurrent.track != "") {
				sResumeFile = oCurrent.track;
			}
			if (oCurrent.position != "") {
				sSeek = oCurrent.position;
			}

            if (szDeviceType == 'RAUMFELD') {
                nTrackNr = oCurrent.trackNR;
                
                var resultMedia = new Object({
                            'status' : '',
                            'data' : '',
                            'dataXML' : ''    
                          });                
                upnpGetMediaContent(nPlayerId, szIp, nPort, avtransport_service_type, resultMedia, curl_options);    
                sResumeFile = resultMedia.data;
                if (resultMedia.dataXML.indexOf("audioBroadcast.radio")>-1) {// only for radio
                    sResumeFileXML = resultMedia.dataXML;
                }
            }
            
            lInfo.logln("upnp-controller exec track:" + sResumeFile + " / position:" + sSeek +" / Volume: " + volume);
        
        } // if (sInfo == 'PLAYING')
        
		// playing event music
		lInfo.logln("upnp-controller scene play:" + szMediaContentURL);
		upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
        var nVolume = -1;
		if (szMediaContentURL.split(';').length>1)
		{
            nVolume = getVolumeFromText(szMediaContentURL);
            szMediaContentURL = getPlaylistFromText(szMediaContentURL);
		} 
        var sPlayUrl = szMediaContentURL;
        
		lInfo.logln('--> URL : ' + szMediaContentURL);
		
        if (nValue < 1)
            nValue = 1000;
        else
            nValue = nValue * 1000;

        var timeoutHandle;
        // Fetch timeoutHandle from the property
        timeoutHandle = loadSubParameter('upnp', 'timer');

        if (szDeviceType != 'RAUMFELD') {
           upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, sPlayUrl, result, curl_options);
           upnpPlay(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
           timeoutHandle = setTimeout(function() {
                upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, parseInt(""+nVolume,10), result, curl_options);
                upnpResumeMusic(szIp, nPort, nValue, nPlayerId, avtransport_service_type, curl_options, sResumeFile, sSeek)
           }, nValue);

        }
        else {
            var nSeekTrack = -1;
            if (sPlayUrl.indexOf('track:')==0) {
              var seekTrack = sPlayUrl.substring(sPlayUrl.indexOf("track:")+6,sPlayUrl.indexOf("*@*"));  
              var tmpTrack = parseInt(seekTrack);
              if (tmpTrack != 'NaN')  
                nSeekTrack = parseInt(tmpTrack);
                
              sPlayUrl = sPlayUrl.substring(sPlayUrl.indexOf("*@*")+3);
            }
            lInfo.logln("!!!!!!!!!!!!!!!!!!!!!!   sPlayUrl="+sPlayUrl+"|");
            if (nVolume!=-1)
            	upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, parseInt(""+nVolume,10), result, curl_options);
        
            
            
            upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, sPlayUrl, result, curl_options);
              
            if (nSeekTrack > 1) { // track selected, reposition to track inside current playlist 
              upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);  
              setTimeout(function() {
                upnpSeekToTrack(nPlayerId, szIp, nPort, nSeekTrack, avtransport_service_type, result, curl_options);  
                timeoutHandle = setTimeout(function() {
                		lInfo.logln('SETTING VOLUMNE ' + parseInt(""+nVolume,10) + ' old value ' + volume);
                        upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, parseInt(""+volume,10), result, curl_options);
                        upnpResumeMusicRaumfeld(szIp, nPort, nValue, nPlayerId, avtransport_service_type, curl_options, sResumeFile, sSeek, sResumeFileXML, nTrackNr,volume);
                   }, nValue);
              }, SEEK_TEUFEL_DELAY); 
            }
            else {
                upnpSeekToTrack(nPlayerId, szIp, nPort, nSeekTrack, avtransport_service_type, result, curl_options);  
                timeoutHandle = setTimeout(function() {
                		lInfo.logln('SETTING VOLUMNE ' + parseInt(""+nVolume,10) + ' old value ' + volume);
                        upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, parseInt(""+volume,10), result, curl_options);
                        upnpResumeMusicRaumfeld(szIp, nPort, nValue, nPlayerId, avtransport_service_type, curl_options, sResumeFile, sSeek, sResumeFileXML, nTrackNr,volume);
                   }, nValue);
            }
        }    

		// Store timeoutHandle to the property
		storeSubParameter('upnp', 'timer', timeoutHandle);
		Property.store();
	} 
    else if (szCmd == UPNP_SET_VOLUME) {
		lInfo.logln("upnp-controller exec SetVolume");
		upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, nValue, result, curl_options);
		lInfo.logln("-----upnp-controller exec SetVolume");
	} else if (szCmd == UPNP_GET_MEDIA_URL_REG_EVENT) {
		lInfo.logln("upnp-controller exec GetMediaRegEvent");
		result.data = upnpGetMediaContent(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
	} else if (szCmd === UPNP_GET_MEDIA_URL) {
		lInfo.logln("upnp-controller exec GetMedia");
		upnpGetMediaContent(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
	} else if (szCmd == UPNP_STOP_RESUME_PLAY) {
		lInfo.logln("upnp-controller exec GetMediaRegEvent");
		upnpGetMediaInfo(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
		upnpGetPositionInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
		upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
		upnpGetTransportSettings(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
		/*
		 * upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result ,
		 * curl_options); upnpPlayStoredPlaylist(nPlayerId, szIp, nPort,
		 * avtransport_service_type , curl_options); upnpSetTimeout(nPlayerId,
		 * szIp, nPort, avtransport_service_type , curl_options);
		 */
	} else if (szCmd == UPNP_GET_MEDIA_URL_POSITION) {
        upnpGetPositionTrackUrl(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
        lInfo.logln("result"+JSON.stringify(result));
        // catch also MediaInfo
        if (szDeviceType == 'RAUMFELD') {
          var resultMedia = new Object({
            'status' : '',
            'data' : '',
            'dataXML' : ''    
          });
          upnpGetMediaContent(nPlayerId, szIp, nPort, avtransport_service_type, resultMedia, curl_options);
          result.mediaInfo = resultMedia;
        }
        
	} else if (szCmd == UPNP_INC_VOLUME) {
		lInfo.logln(nPlayerId + "/" + szIp + "/" + nPort + "/" + avtransport_service_type + "/" + result);
		upnpSetPlayerVolume(1, nPlayerId, szIp, nPort, render_service_type, result, curl_options);
	} else if (szCmd == UPNP_DEC_VOLUME) {
		upnpSetPlayerVolume(0, nPlayerId, szIp, nPort, render_service_type, result, curl_options);
	}
	lInfo.logln("-----upnp-controller exec SetVolume2");
	return result;
}

function upnpSetPlayerVolume(nVolumeType, nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPACTION: "urn:schemas-upnp-org:service:RenderingControl:1#GetVolume"';
		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetVolume xmlns:u="urn:schemas-upnp-org:service:' + 'RenderingControl' + ':1' + '">' + '<InstanceID>0</InstanceID>' + '<Channel>Master</Channel>' + '</u:GetVolume>'
				+ '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
        lInfo.logln(url+" - "+action);
        lInfo.logln(JSON.stringify(result));
		var nVolume = parseInt(upnpParseXMLRespond(result.body, 'CurrentVolume', ''));
		lInfo.logln("StartVolume:" + nVolume);

		var bChangeVolume = true;
		if (nVolumeType == 1) { // increase
			if (nVolume > 99)
				bChangeVolume = false;
			nVolume = nVolume + UPNP_VOLUME_STEP;
			if (nVolume > 100)
				nVolume = 100;
		} else { // decrease
			if (nVolume < 1)
				bChangeVolume = false;
			nVolume = nVolume - UPNP_VOLUME_STEP;
			if (nVolume < 0)
				nVolume = 0;
		}

		if (bChangeVolume) {
			upnpSetVolume(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, nVolume, result);
		}

		lInfo.logln(url);
		lInfo.logln(action);
	} catch (e) {
		lWarn.logln("Error executing upnpSetVolume command " + e);
	}
}

function upnpGetProtocolInfo(nPlayerId, szIpAddressRenderer, nPortRenderer, nValue, szServiceType, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction: ' + '"urn:schemas-upnp-org:service:' + 'ConnectionManager' + ':1' + '#' + 'GetProtocolInfo"'
		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + '/ConnectionManager/ctrl';

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetProtocolInfo xmlns:u="urn:schemas-upnp-org:service:' + 'ConnectionManager' + ':1' + '">' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
	} catch (e) {
		lWarn.logln("Error executing upnpGetProtocolInfo command " + e);
	}
}

/**
 * @brief Set mute
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param nValue -
 *            0 -> perform unmute, 1 -> perform mute
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpSetMute(nPlayerId, szIpAddressRenderer, nPortRenderer, nValue, szServiceType, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction: ' + '"urn:schemas-upnp-org:service:' + 'RenderingControl' + ':1'
		// + szServiceType + ':1'
		+ '#' + 'SetMute"'
		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:SetMute xmlns:u="urn:schemas-upnp-org:service:' + 'RenderingControl' + ':1' + '">' + '<InstanceID>0</InstanceID>' + '<Channel>Master</Channel>' + '<DesiredMute>'
				+ nValue + '</DesiredMute>' + '</u:SetMute>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
		lInfo.logln(url);
		lInfo.logln(action);
	} catch (e) {
		lWarn.logln("Error executing upnpSetMute command " + e);
	}
}

/**
 * @brief Set pause
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param nValue -
 *            not used
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpSetPause(nPlayerId, szIpAddressRenderer, nPortRenderer, nValue, szServiceType, oResult, oCurlOptions) {

	try {
		lInfo.logln("PAUSE STARTING");

		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction: ' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1' + '#' + 'Pause"'
		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:Pause xmlns:u="urn:schemas-upnp-org:service:' + 'AVTransport' + ':1' + '">' + '<InstanceID>0</InstanceID>' + '</u:Pause>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
	} catch (e) {
		lWarn.logln("Error executing upnpSetPause command " + e);
	}
}

function upnpSeek(nPlayerId, szIpAddressRenderer, nPortRenderer, nValue, szServiceType, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction: ' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1' + '#' + 'Seek"'
		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:Seek xmlns:u="urn:schemas-upnp-org:service:' + 'AVTransport' + ':1' + '">' + '<InstanceID>0</InstanceID>' + '<Unit>REL_TIME</Unit>' + '<Target>' + nValue
				+ '</Target>' + '</u:Seek>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
	} catch (e) {
		lWarn.logln("Error executing upnpSeek command " + e);
	}
}

function upnpSeekToTrack(nPlayerId, szIpAddressRenderer, nPortRenderer, nValue, szServiceType, oResult, oCurlOptions) {
    try {
        var http_headers = [];
        http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
        http_headers['extra_hdr2'] = 'SOAPAction: ' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1' + '#' + 'Seek"'
        var h = new HTTP(oCurlOptions, http_headers);

        h.reset();

        var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

        var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
                + '<s:Body>' + '<u:Seek xmlns:u="urn:schemas-upnp-org:service:' + 'AVTransport' + ':1' + '">' + '<InstanceID>0</InstanceID>' + '<Unit>TRACK_NR</Unit>' + '<Target>' + nValue
                + '</Target>' + '</u:Seek>' + '</s:Body>' + '</s:Envelope>';

        var result = h.post(url, '', action);
    } catch (e) {
        lWarn.logln("Error executing upnpSeekToTrack command " + e);
    }
}

/**
 * @brief Set resume
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param nValue -
 *            not used
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpSetResume(nPlayerId, szIpAddressRenderer, nPortRenderer, nValue, szServiceType, oResult, oCurlOptions) {

	try {
        lWarn.logln("******Start resuming*******");
		var result = upnpPlay(szIpAddressRenderer, nPortRenderer, szServiceType, oCurlOptions);
	} catch (e) {
		lWarn.logln("Error executing upnpSetResume command " + e);
	}
}

/**
 * @brief Perform stop
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpStop(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'Stop"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:Stop xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:Stop>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
	} catch (e) {
		lWarn.logln("Error executing upnpStop command " + e + " [url=" + url + "]");
	}
}

/**
 * @brief Perform upnp server source selection
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param szUpnpServerContent -
 *            full path to the upnp server content
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpSetContentSrc(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, szUpnpServerContent, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';//
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'SetAVTransportURI"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

        var sValue = '<CurrentURI>' + xmlEscape(szUpnpServerContent) + '</CurrentURI><CurrentURIMetaData />';
        if (szUpnpServerContent.indexOf('DIDL-Lite') > -1)
          sValue = '<CurrentURI></CurrentURI><CurrentURIMetaData>'+ xmlEscapeURI(szUpnpServerContent) + '</CurrentURIMetaData>';
        lInfo.logln("upnpSetContentSrc="+sValue);
        
		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:SetAVTransportURI xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + sValue  
				+ '</u:SetAVTransportURI>' + '</s:Body>' + '</s:Envelope>';
        lInfo.logln(action);
		var result = h.post(url, '', action);

		lInfo.logln(url);
		lInfo.logln(action);
	} catch (e) {
		lWarn.logln("Error executing upnpSetContentSrc command " + e);
	}
}

/**
 * @brief Perform play
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpPlay(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'Play"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:Play xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '<Speed>1</Speed>' + '</u:Play>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
	} catch (e) {
		lWarn.logln("Error executing upnpPlay command " + e);
	}
}

/**
 * @brief Perform upnp set volume
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param nVolumeLevel -
 *            volume level
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpSetVolume(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, nVolumeLevel, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'RenderingControl' + ':1#' + 'SetVolume"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:SetVolume xmlns:u="urn:schemas-upnp-org:service:RenderingControl:1">' + '<InstanceID>0</InstanceID>' + '<Channel>Master</Channel>' + '<DesiredVolume>'
				+ nVolumeLevel + '</DesiredVolume>' + '</u:SetVolume>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
        lInfo.logln("SET VOLUME -------------");
		lInfo.logln(url);
		lInfo.logln(action);
        lInfo.logln("SET VOLUME -------------");
	} catch (e) {
		lWarn.logln("Error executing upnpSetVolume command " + e);
	}
}

/**
 * @brief Perform get currently played media content src from upnp renderer
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpGetMediaContent(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'GetMediaInfo"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetMediaInfo xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetMediaInfo>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

        result.body = htmlUnescape(result.body);
        
		/* Set result status */
		oResult.status = result.status;
		oResult.data = upnpParseXMLRespond(result.body, 'CurrentURI', '');
        oResult.dataXML = cleanXMLText(upnpParseXMLRespond(result.body, 'CurrentURIMetaData', ''));

		lInfo.logln("Extracted media content: " + oResult.data);
        lInfo.logln("Extracted media org: " + result.body);
	} catch (e) {
		lWarn.logln("Error executing upnpGetMediaContent command " + e);
	}
}

/**
 * @brief Get upnp volume
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpGetVolume(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'RenderingControl' + ':1#' + 'GetVolume"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetVolume xmlns:u="urn:schemas-upnp-org:service:RenderingControl:1">' + '<InstanceID>0</InstanceID>' 
				+ '<Channel>Master</Channel>' + '</u:GetVolume>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		/* Set result status */
		oResult.status = result.status;
		oResult.data = upnpParseXMLRespond(result.body, 'CurrentVolume', '');

		
		lInfo.logln(url);
		lInfo.logln("Volume is " + oResult.data);
		return oResult.data;
	} catch (e) {
		lWarn.logln("Error executing upnpGetContentSrc command " + e);
	}
	return null;
}


/**
 * @brief Retrieve media info from the player and store it into the temp storage
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpGetMediaInfo(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'GetMediaInfo"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetMediaInfo xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetMediaInfo>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		var oReturn = new Object();
		oReturn.nrTracks = upnpParseXMLRespond(result.body, 'NrTracks', '');
		return oReturn;
	} catch (e) {
		lWarn.logln("Error executing upnpGetMediaInfo command " + e);
	}
	return null;
}

/*
 * function upnpLoadSonosPlayList(nPlayerId, szIpAddressRenderer, nPortRenderer ,
 * szServiceType, oResult, oCurlOptions) { try { var http_headers = [];
 * http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
 * http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' +
 * 'AVTransport' + ':1#' + 'AddURIToQueue"';
 * 
 * var h = new HTTP(oCurlOptions, http_headers);
 * 
 * h.reset();
 * 
 * var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer +
 * szServiceType; var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope
 * s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
 * xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">' + '<s:Body>' + '<u:AddURIToQueue
 * xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '<EnqueuedURI>file:///jffs/settings/savedqueues.rsq#12</EnqueuedURI>' + '<EnqueuedURIMetaData>&lt;DIDL-Lite
 * xmlns:dc=&quot;http://purl.org/dc/elements/1.1/&quot;
 * xmlns:upnp=&quot;urn:schemas-upnp-org:metadata-1-0/upnp/&quot;
 * xmlns:r=&quot;urn:schemas-rinconnetworks-com:metadata-1-0/&quot;
 * xmlns=&quot;urn:schemas-upnp-org:metadata-1-0/DIDL-Lite/&quot;&gt;&lt;item
 * id=&quot;SQ:12&quot; parentID=&quot;SQ:&quot;
 * restricted=&quot;true&quot;&gt;&lt;dc:title&gt;Amb Drum &amp;amp;
 * Bass&lt;/dc:title&gt;&lt;upnp:class&gt;object.container.pla
 * ylist&lt;/upnp:class&gt;&lt;desc id=&quot;cdudn&quot;
 * nameSpace=&quot;urn:schemas-rinconnetworks-com:metadata-1-0/&quot;&gt;RINCON_AssociatedZPUDN&lt;/desc&gt;&lt;/item&gt;&lt;/DIDL-Lite&gt;</EnqueuedURIMetaData>' + '<DesiredFirstTrackNumberEnqueued>0</DesiredFirstTrackNumberEnqueued>' + '<EnqueueAsNext>1</EnqueueAsNext>' + '</u:AddURIToQueue>' + '</s:Body>' + '</s:Envelope>'
 * 
 * var result = h.post(url, '', action);
 * 
 * return result; } catch (e) { lInfo.logln("Error executing
 * upnpLoadSonosPlayList command " + e); } } function
 * upnpSaveSonosPlayList(nPlayerId, szIpAddressRenderer, nPortRenderer ,
 * szServiceType, oResult, oCurlOptions) { try { var http_headers = [];
 * http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
 * http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' +
 * 'AVTransport' + ':1#' + 'SaveQueue"';
 * 
 * var h = new HTTP(oCurlOptions, http_headers);
 * 
 * h.reset();
 * 
 * var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer +
 * szServiceType;
 * 
 * var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope
 * s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
 * xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">' + '<s:Body>' + '<u:SaveQueue
 * xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '<Title>Test</Title>' + '<ObjectID></ObjectID>' + '</u:SaveQueue>' + '</s:Body>' + '</s:Envelope>'
 * 
 * var result = h.post(url, '', action);
 * 
 * return upnpParseXMLRespond(result.body, 'AssignedObjectID', '') } catch (e) {
 * lInfo.logln("Error executing upnpSaveSonosPlayList command " + e); } }
 * function upnpGetPlaylistSongs(nPlayerId, szIpAddressServer, nPortServer ,
 * sPlaylistCode, oCurlOptions) { try { var http_headers = [];
 * http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
 * http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' +
 * 'ContentDirectory' + ':1#' + 'Browse"';
 * 
 * 
 * var h = new HTTP(oCurlOptions, http_headers);
 * 
 * h.reset();
 * 
 * var url = 'http://' + szIpAddressServer + ':' + nPortServer +
 * "/_urn-schemas-upnp-org-service-ContentDirectory_control"; var action = '<?xml
 * version="1.0" encoding="utf-8"?>' + '<s:Envelope
 * s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
 * xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">' + '<s:Body>' + '<u:Browse
 * xmlns:u="urn:schemas-upnp-org:service:ContentDirectory:1">' + '<ObjectID>0000000000000005</ObjectID>' + '<BrowseFlag>BrowseDirectChildren</BrowseFlag>' + '<Filter>@childCount,dc:creator,upnp:album,upnp:artist,res,res@protection,res@duration,res@size,upnp:albumArtURI,upnp:searchClass</Filter>' + '<StartingIndex>0</StartingIndex>' + '<RequestedCount>0</RequestedCount>' + '<SortCriteria
 * />' + '</u:Browse>' + '</s:Body>' + '</s:Envelope>';
 * 
 * var result = h.post(url, '', action);
 * 
 * var oReturn = new Object(); oReturn.result = upnpParseXMLRespond(result.body,
 * 'Result', ''); oReturn.resultCount = upnpParseXMLRespond(result.body,
 * 'NumberReturned', '');
 * 
 * return oReturn; } catch (e) { lInfo.logln("Error executing
 * upnpGetPlaylistSongs command " + e); } }
 */

/**
 * @brief Retrieve position info from the player and store it into the temp
 *        storage
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpGetPositionInfo(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'GetPositionInfo"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetPositionInfo xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetPositionInfo>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		var oReturn = new Object();
		oReturn.track = upnpParseXMLRespond(result.body, 'TrackURI', '');
		oReturn.position = upnpParseXMLRespond(result.body, 'RelTime', '');
		oReturn.trackNR = upnpParseXMLRespond(result.body, 'Track', '');

		return oReturn;
	} catch (e) {
		lWarn.logln("Error executing upnpGetPositionInfo command " + e);
	}
	return null;
}

function upnpGetPositionTrackUrl(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'GetPositionInfo"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetPositionInfo xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetPositionInfo>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		oResult.status = result.status;
		oResult.data = upnpParseXMLRespond(result.body, 'TrackURI', '');
        oResult.track = upnpParseXMLRespond(result.body, 'Track', '');
		lInfo.logln("media info url: " + oResult.data);

	} catch (e) {
		lWarn.logln("Error executing upnpGetPositionInfo command " + e);
	}
}

/**
 * @brief Retrieve transport info from the player and store it into the temp
 *        storage
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpGetTransportInfo(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'GetTransportInfo"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetTransportInfo xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetTransportInfo>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		return upnpParseXMLRespond(result.body, 'CurrentTransportState', '')
	} catch (e) {
		lWarn.logln("Error executing upnpGetTransportInfo command " + e);
	}
	return null;
}

/**
 * @brief Retrieve transport settings from the player and store it into the temp
 *        storage
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpGetTransportSettings(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'GetTransportSettings"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetTransportSettings xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetTransportSettings>' + '</s:Body>'
				+ '</s:Envelope>';

		var result = h.post(url, '', action);
	} catch (e) {
		lWarn.logln("Error executing upnpGetTransportSettings command " + e);
	}
}

